
# coding: utf-8

# In[1]:


import numpy as np
import matplotlib.pyplot as plt
from math import sin, asin, pi
get_ipython().run_line_magic('matplotlib', 'inline')
from astropy.io import fits
import astropy.visualization as visual


# In[2]:


class fits_picture(object):
    
    def __init__(self, name):
        self.name = name
        with fits.open(self.name) as file:
            self.data = file[0].data
        self.crop_data = self.data
        self.values = []
        self.indexes = []
        self.x = []
        self.lambdas = []
    
    def show_picture(self, cropped=False):
        data = self.data
        if cropped:
            data = self.crop_data
        percentile = visual.PercentileInterval(99.)
        plt.set_cmap('Greys')
        plt.imshow(percentile(data))
        
    def crop_picture(self, y_min, y_max, x_min, x_max):
        self.crop_data = self.data[y_min:y_max, x_min:x_max]
        
    def get_values(self, precision, diagonal):
        if diagonal:
            minimum = min(len(self.crop_data), len(self.crop_data[0])) - precision
            values = [0] * minimum
            for i in range(minimum):
                for n in range(precision):
                    values[minimum - i - 1] += self.crop_data[i, i + n] / precision
        else:
            center = len(self.crop_data[0]) // 2
            values = [0] * len(self.crop_data[0])
            for i in range(len(self.crop_data)):
                for n in range(precision):
                    values[len(self.crop_data) - i - 1] += self.crop_data[i, center + n] / precision
        return values
        
    def make_graph(self, precision=5, diagonal=False):
        self.values = self.get_values(precision,diagonal)
        plt.figure(figsize=(7,4.5))
        plt.text(400, 11000, '1', fontsize=13)
        plt.text(770, 26000, '2', fontsize=13)
        plt.text(1083, 63000, '3', fontsize=13)
        plt.text(1373, 18000, '4', fontsize=13)
        plt.text(1602, 16000, '5', fontsize=13)
        plt.text(1800, 5500, '6', fontsize=13)
        plt.plot(np.linspace(0, len(self.values), len(self.values)), self.values)
        plt.savefig('speky.png')
        
    def find_index_of_maxs(self, list_of_borders):
        list_of_indexes = []
        for lower, upper in list_of_borders:
            list_of_indexes.append(self.values.index(max(self.values[lower:upper])))
        self.indexes = list_of_indexes
        print(self.indexes)
        
    def linear_fit(self, list_of_lambdas): # y = A + B * x
        self.lambdas = list_of_lambdas
        #with open('data_indexy.txt', 'w') as file:
        #    for i in range(len(self.lambdas)):
        #        string = str(self.indexes[i]) +' ' + '{:.1f}'.format(self.lambdas[i]).replace('.', ',') + '\n'
        #        file.write(string)
        return np.polyfit(self.indexes, list_of_lambdas, 1)
        
    def make_final_graph(self, list_of_lambdas):
        B, A = self.linear_fit(list_of_lambdas)
        x = []
        for i in range(len(self.values)):
            x.append(A + B*i)
        self.x = x
        plt.figure(figsize=(7,4.5))
        plt.plot(x,self.values)
        plt.xlim(400,670)
        plt.ylim(0,70000)
        plt.xlabel('$\lambda$ [nm]', size=12)
        plt.ylabel('I [counts]', size=12)
        plt.title('Sepktrum zářivky',size=16)
        plt.savefig('spek.png')
    
    def make_txt(self):
        with open('data.txt', 'w') as file:
            for i in range(len(self.x)):
                x = self.x[i]
                y = self.values[i]
                file.write(str(x) + ' ' + str(y) + '\n')
                
    def make_txt_maxs(self):
        with open('data_maxs.txt', 'w') as file:
            for i in range(len(self.lambdas)):
                x = self.indexes[i]
                y = self.lambdas[i]
                file.write(str(x) + ' ' + str(y) + '\n')


# In[3]:


pic = fits_picture('tomas_plsek.fits') # vytvoří objekt, na němž voláme jednotlivé funkce
pic.show_picture()                     # otřevřete si obrázek a vyberete rozumnou oblast snímku


# In[4]:


pic.crop_picture(1300, 3300, 1500, 3500) # oříznete snímek (vždy pouze původní snímek)
pic.show_picture(True)                   # zobrazí ořezaný snímek


# In[5]:


pic.make_graph(10, False)                 # vytvoří jednoduchý graf, první parametr uvává přesnost
                                         # druhý parametr uvádá, zda se má obrázek procházet diagonálně True/False


# In[6]:


pic.find_index_of_maxs([[350,450], [700,800], [1150, 1250], [1350, 1450], [1500, 1700], [1730, 1830]])
# na snímku naleznete očima hledané intervaly, vnichž si myslíte, že se maxima nacházejí
# výpis je pro kontrolu (nemusíte získané hodnoty nikam psát kód s nimi pracuje)


# In[7]:


pic.make_final_graph([436.6, 487.7, 542.4, 584, 611.6, 650.8])
# funkci make_final_graph dáte seznam vlnových délek odpovídají vašim maximům
# doporučuji buď alespoň změnit parametry grafu - ve funkci make_new_graph
# případně odkomentovat následující řádek, který vám vyhodí texťák získáných hodnot s názvem 'data.txt'
# pic.make_txt()
pic.make_txt_maxs()


# In[46]:


def get_distance(height, dist, lambdas):
    sin_alpha = []
    delka = []
    for i in range(len(dist)):
        sin_alpha.append(dist[i] / (height**2 + dist[i]**2)**.5)
        delka.append(lambdas[i] / sin_alpha[i])
        print('{:.3f}'.format(lambdas[i] / sin_alpha[i]))
    with open('delka_vrypu.txt', 'w') as file:
        for i in range(len(dist)):
            string = str(lambdas[i]) + ' ' + '{:.2f}'.format(dist[i]).replace('.', ',') +             ' ' + '{:.1f}'.format(asin(sin_alpha[i]) * 57.3) + ' ' +             '{:.1f}'.format(delka[i]).replace('.', ',') + '\n'
            file.write(string)
    mean, stdev = np.mean(delka), np.std(delka)
    print('{:.2f} +/- {:.2f}'.format(mean, stdev/4**.5))
    print('{:.2f} +/- {:.2f}'.format(asin(sin_alpha[1]) * 57.3, np.std([asin(sin_alpha[0]) * 57.3, asin(sin_alpha[1]) * 57.3, asin(sin_alpha[2]) * 57.3])))


# In[47]:


get_distance(1.7, [0.75, 0.8, 0.85], [650, 650, 650])


# In[21]:


mega_list = []
for i in range(4):
    small_list = [1] * 5
    for j in range(8):
        small_list[i] = j + 1
        for n in range(8):
            small_list[i+1] = n + 1
            print(small_list, sorted(set(small_list)))
            if small_list == sorted(set(small_list)):
                mega.list.append(small_list)
mega_list

